/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <dlog.h>

#include "$(appName).h"
#include "data.h"

#define FAVOURITES_DATA_COUNT 10
#define MY_DATA_COUNT 1
#define DATA_COUNT 50

typedef struct _contact_data_s {
	char *name;
	char *number;
	char *img_path;
} contact_data_s;

static contact_data_s favourites_data[FAVOURITES_DATA_COUNT] = {
	{ "Boyeon Son", "010-2345-6789", "images/img1.png" },
	{ "Geunsun Lee", "010-1234-5678", NULL },
	{ "Junkyu Han", "010-3456-7891", NULL },
	{ "Moonkyung Park", "010-4567-8910", NULL },
	{ NULL, NULL, NULL },
};

static contact_data_s my_data[MY_DATA_COUNT] = {
	{ "Eunyoung Lee", "010-5678-9101", "images/img2.png" },
};

static contact_data_s contact_data[DATA_COUNT] = {
	{ "Chulseung Kim", "010-5566-7788", NULL },
	{ "Heonjae Jang", "010-7700-9900", NULL },
	{ "Hyerim Kim", "010-3455-2784", NULL },
	{ "Hyoeun Ahn", "010-7891-0111", "images/img3.png" },
	{ "Jinyoung Kim", "010-0023-0210", NULL },
	{ "Juhee Park", "010-9101-1121", "images/img4.png" },
	{ "Seokhyun Kang", "010-1112-1314", NULL },
	{ "Seongmoon Jeong", "010-8899-0011", NULL },
	{ "Kyuho Cho", "010-0088-8979", NULL },
	{ "Soofeel Kim", "010-8910-1112", NULL },
	{ "Sungjae Park", "010-1011-2341", NULL },
	{ "Yongkook Kim", "010-7788-9900", NULL },
	{ "Younghun Cho", "010-1011-1213", NULL },
	{ "Youngjoo Park", "010-6789-1011", NULL },
	{ NULL, NULL, NULL },
};

/**
 * @brief Initialization function for data module.
 */
void data_initialize(void)
{
	/**
	 * If you need to initialize managing data,
	 * please use this function.
	 */
}

/**
 * @brief Finalization function for data module.
 */
void data_finalize(void)
{
	/**
	 * If you need to finalize managing data,
	 * please use this function.
	 */
}

/**
 * @brief Gets a text.
 * @param[part]: Name string of one of the existing text parts in the EDJ group
 */
char *data_get_text(const char *part)
{
	/**
	 * You can use this function to retrieve data.
	 */
	return NULL;
}

/**
 * @brief Function will be operated when genlist's items are shown on the screen.
 * @param[in] data The data passed from 'elm_genlist_item_append' as third parameter
 * @param[in] obj The genlist object
 * @param[in] part Name string of one of the existing text parts in the EDJ group implementing the item's theme
 */
char *data_get_gl_title_text(void *data, Evas_Object *obj, const char *part)
{
	char buf[BUF_LEN];

	snprintf(buf, sizeof(buf), "%s", "Contacts");
	return strdup(buf);
}

/**
 * @brief Function will be operated when genlist's items are shown on the screen.
 * @param[in] data The data passed from 'elm_genlist_item_append' as third parameter
 * @param[in] obj The genlist object
 * @param[in] part Name string of one of the existing text parts in the EDJ group implementing the item's theme
 */
char *data_get_gl_type_text(void *data, Evas_Object *obj, const char *part)
{
	char buf[BUF_LEN];
	contact_type_e type = (contact_type_e) data;

	switch (type) {
	case CONTACT_TYPE_FAVOURITES:
		snprintf(buf, sizeof(buf), "%s", "Favourites");
		break;
	case CONTACT_TYPE_MY_PROFILE:
		snprintf(buf, sizeof(buf), "%s", "My profile");
		break;
	case CONTACT_TYPE_CONTACTS:
		snprintf(buf, sizeof(buf), "%s", "#");
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "This contact has the wrong information of type.");
		snprintf(buf, sizeof(buf), "%s", "#");
		break;
	}

	return strdup(buf);
}

/**
 * @brief Function will be operated when genlist's items are shown on the screen.
 * @param[in] data The data passed from 'elm_genlist_item_append' as third parameter
 * @param[in] obj The genlist object
 * @param[in] part Name string of one of the existing text parts in the EDJ group implementing the item's theme
 */
char *data_get_gl_text(void *data, Evas_Object *obj, const char *part)
{
	char buf[BUF_LEN];
	contact_info_s *contact_info = (contact_info_s *) data;
	contact_type_e type = contact_info->type;
	int i = contact_info->index;

	switch (type) {
	case CONTACT_TYPE_FAVOURITES:
		snprintf(buf, sizeof(buf), "%s", favourites_data[i].name);
		break;
	case CONTACT_TYPE_MY_PROFILE:
		snprintf(buf, sizeof(buf), "%s", my_data[i].name);
		break;
	case CONTACT_TYPE_CONTACTS:
		snprintf(buf, sizeof(buf), "%s", contact_data[i].name);
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "This contact has the wrong information of type.");
		break;
	}

	return strdup(buf);
}

/**
 * @brief Gets count of contacts to display according to type.
 * @param[in] type Identify the type of contacts
 */
int data_get_contact_count(const contact_type_e type)
{
	int count = 0;
	int i = 0;

	switch (type) {
	case CONTACT_TYPE_FAVOURITES:
		for (i = 0; i < FAVOURITES_DATA_COUNT; i++) {
			if (favourites_data[i].name == NULL) {
				break;
			}
			count++;
		}
		break;
	case CONTACT_TYPE_MY_PROFILE:
		count = MY_DATA_COUNT;
		break;
	case CONTACT_TYPE_CONTACTS:
		for (i = 0; i < DATA_COUNT; i++) {
			if (contact_data[i].name == NULL) {
				break;
			}
			count++;
		}
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "This contact has the wrong information of type.");
		break;
	}

	return count;
}

/**
 * @brief Gets a name from contacts data.
 * @param[in] type Identify the type of contacts
 * @param[in] index The order of contacts
 */
char *data_get_contact_name(const contact_type_e type, int index)
{
	char buf[128] = { 0, };

	switch (type) {
	case CONTACT_TYPE_FAVOURITES:
		snprintf(buf, sizeof(buf), "%s", favourites_data[index].name);
		break;
	case CONTACT_TYPE_MY_PROFILE:
		snprintf(buf, sizeof(buf), "%s", my_data[index].name);
		break;
	case CONTACT_TYPE_CONTACTS:
		snprintf(buf, sizeof(buf), "%s", contact_data[index].name);
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "This contact has the wrong information of type.");
		break;
	}

	return strdup(buf);
}

/**
 * @brief Gets a call number from contacts data.
 * @param[in] type Identify the type of contacts
 * @param[in] index The order of contacts
 */
char *data_get_contact_number(const contact_type_e type, int index)
{
	char buf[128] = { 0, };

	switch (type) {
	case CONTACT_TYPE_FAVOURITES:
		snprintf(buf, sizeof(buf), "%s", favourites_data[index].number);
		break;
	case CONTACT_TYPE_MY_PROFILE:
		snprintf(buf, sizeof(buf), "%s", my_data[index].number);
		break;
	case CONTACT_TYPE_CONTACTS:
		snprintf(buf, sizeof(buf), "%s", contact_data[index].number);
	break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "This contact has the wrong information of type.");
		break;
	}

	return strdup(buf);
}

/**
 * @brief Gets a image path from contacts data.
 * @param[in] type Identify the type of contacts
 * @param[in] index The order of contacts
 */
char *data_get_contact_image(const contact_type_e type, int index)
{
	char buf[PATH_MAX] = { 0, };

	switch (type) {
	case CONTACT_TYPE_FAVOURITES:
		if (favourites_data[index].img_path == NULL) {
			return NULL;
		} else {
			data_get_resource_path(favourites_data[index].img_path, buf, (int)PATH_MAX);
		}
		break;
	case CONTACT_TYPE_MY_PROFILE:
		if (my_data[index].img_path == NULL) {
			return NULL;
		} else {
			data_get_resource_path(my_data[index].img_path, buf, (int)PATH_MAX);
		}
		break;
	case CONTACT_TYPE_CONTACTS:
		if (contact_data[index].img_path == NULL) {
			return NULL;
		} else {
			data_get_resource_path(contact_data[index].img_path, buf, (int)PATH_MAX);
		}
		break;
	default:
		dlog_print(DLOG_ERROR, LOG_TAG, "This contact has the wrong information of type.");
		break;
	}

	return strdup(buf);
}

/**
 * @brief Gets path of resource.
 * @param[in] file_in File name
 * @param[in] file_path_out The point to which save full path of the resource
 * @param[in] file_path_max Size of file name include path
 */
void data_get_resource_path(const char *file_in, char *file_path_out, int file_path_max)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(file_path_out, file_path_max, "%s%s", res_path, file_in);
		free(res_path);
	}
}

/* End of file */
